@@ -48,6 +48,12 @@
  #define PF_XDP AF_XDP
 #endif
 
+#ifndef SO_NETNS_COOKIE
+ #define SO_NETNS_COOKIE 71
+#endif
+
+#define INIT_NS 1
+
 struct xsk_umem {
 	struct xsk_ring_prod *fill_save;
 	struct xsk_ring_cons *comp_save;
@@ -67,9 +73,11 @@ struct xsk_ctx {
 	__u32 queue_id;
 	int refcount;
 	int ifindex;
+	__u64 netns_cookie;
 	int xsks_map_fd;
 	struct list_head list;
 	struct xdp_program *xdp_prog;
+	int refcnt_map_fd;
 	char ifname[IFNAMSIZ];
 };
 
@@ -380,12 +388,10 @@ static int xsk_init_xsk_struct(struct xsk_socket *xsk, int ifindex)
 	return 0;
 }
 
-#define XDP_FLAGS_SUPPORTED (XDP_FLAGS_SKB_MODE | XDP_FLAGS_DRV_MODE | XDP_FLAGS_HW_MODE)
-
 static enum xdp_attach_mode xsk_convert_xdp_flags(__u32 xdp_flags)
 {
-	if (xdp_flags & ~XDP_FLAGS_SUPPORTED)
-		pr_warn("XDP flag not supported by libxdp.\n");
+	if (xdp_flags & ~XDP_FLAGS_MASK)
+		pr_warn("XDP flag: 0x%x contains flags not supported by libxdp.\n", xdp_flags);
 
 	if (xdp_flags & XDP_FLAGS_SKB_MODE)
 		return XDP_MODE_SKB;
@@ -495,7 +501,8 @@ static void xsk_delete_map_entry(int xsks_map_fd, __u32 queue_id)
 	close(xsks_map_fd);
 }
 
-static int xsk_lookup_bpf_map(int prog_fd)
+static int xsk_lookup_map_by_filter(int prog_fd,
+				    bool (*map_info_filter)(struct bpf_map_info *map_info))
 {
 	__u32 i, *map_ids, num_maps, prog_len = sizeof(struct bpf_prog_info);
 	__u32 map_len = sizeof(struct bpf_map_info);
@@ -535,8 +542,7 @@ static int xsk_lookup_bpf_map(int prog_fd)
 			continue;
 		}
 
-		if (!strncmp(map_info.name, "xsks_map", sizeof(map_info.name)) &&
-		    map_info.key_size == 4 && map_info.value_size == 4) {
+		if (map_info_filter(&map_info)) {
 			xsks_map_fd = fd;
 			break;
 		}
@@ -548,6 +554,51 @@ static int xsk_lookup_bpf_map(int prog_fd)
 	return xsks_map_fd;
 }
 
+static bool xsk_map_is_socket_map(struct bpf_map_info *map_info)
+{
+	return !strncmp(map_info->name, "xsks_map", sizeof(map_info->name)) &&
+		map_info->key_size == 4 && map_info->value_size == 4;
+}
+
+static bool xsk_map_is_refcnt_map(struct bpf_map_info *map_info)
+{
+	/* In order to avoid confusing users with multiple identically named
+	 * maps, libbpf names non-custom internal maps (.data, .bss, etc.)
+	 * in an unexpected way, namely the first 8 characters of a bpf object
+	 * name + a suffix signifying the internal map type,
+	 * ex. "xdp_def_" + ".data".
+	 */
+	return !strncmp(map_info->name, "xsk_def_.data",
+			sizeof(map_info->name)) &&
+			map_info->value_size >= sizeof(int);
+}
+
+static int xsk_lookup_bpf_map(int prog_fd)
+{
+	return xsk_lookup_map_by_filter(prog_fd, &xsk_map_is_socket_map);
+}
+
+static int xsk_lookup_refcnt_map(int prog_fd, const char *xdp_filename)
+{
+	int map_fd = xsk_lookup_map_by_filter(prog_fd, &xsk_map_is_refcnt_map);
+
+	if (map_fd >= 0)
+		goto out;
+
+	if (map_fd != -ENOENT) {
+		pr_debug("Error getting refcount map: %s\n", strerror(-map_fd));
+		goto out;
+	}
+
+	if (xdp_filename)
+		pr_warn("Refcount was not found in %s or kernel does not support required features, so automatic program removal on unload is disabled\n",
+			xdp_filename);
+	else
+		pr_warn("Another XSK socket was created by a version of libxdp that doesn't support program refcnt, so automatic program removal on unload is disabled.\n");
+out:
+	return map_fd;
+}
+
 #ifdef HAVE_LIBBPF_BPF_MAP_CREATE
 /* bpf_map_create() and the new bpf_prog_create() were added at the same time -
  * however there's a naming conflict with another bpf_prog_load() function in
@@ -645,6 +696,7 @@ static struct xdp_program *xsk_lookup_program(int ifindex)
 
 	if (xdp_multiprog__is_legacy(multi_prog)) {
 		prog = xdp_multiprog__main_prog(multi_prog);
+		prog = strcmp(xdp_program__name(prog), prog_name) ? NULL : prog;
 		goto check;
 	}
 
@@ -652,10 +704,10 @@ static struct xdp_program *xsk_lookup_program(int ifindex)
 		if (!strcmp(xdp_program__name(prog), prog_name))
 			break;
 
+check:
 	if (!prog)
 		goto out;
 
-check:
 	err = check_xdp_prog_version(xdp_program__btf(prog), version_name, &version);
 	if (err) {
 		prog = ERR_PTR(err);
@@ -669,18 +721,75 @@ check:
 
 out:
 	if (!IS_ERR_OR_NULL(prog))
-		prog = xdp_program__clone(prog);
+		prog = xdp_program__clone(prog, 0);
 
 	xdp_multiprog__close(multi_prog);
 	return prog;
 }
 
+static int xsk_update_prog_refcnt(int refcnt_map_fd, int delta)
+{
+	struct bpf_map_info map_info = {};
+	__u32 info_len = sizeof(map_info);
+	int *value_data = NULL;
+	int lock_fd, ret;
+	__u32 key = 0;
+
+	ret = bpf_obj_get_info_by_fd(refcnt_map_fd, &map_info, &info_len);
+	if (ret)
+		return ret;
+
+	value_data = calloc(1, map_info.value_size);
+	if (!value_data)
+		return -ENOMEM;
+
+	lock_fd = xdp_lock_acquire();
+	if (lock_fd < 0) {
+		ret = lock_fd;
+		goto out;
+	}
+
+	/* Note, if other global variables are added before the refcnt,
+	 * this changes map's value type, not number of elements,
+	 * so additional offset must be applied to value_data,
+	 * when reading refcount, but map key always stays zero
+	 */
+	ret = bpf_map_lookup_elem(refcnt_map_fd, &key, value_data);
+	if (ret)
+		goto unlock;
+
+	/* If refcount is 0, program is awaiting detach and can't be used */
+	if (*value_data) {
+		*value_data += delta;
+		ret = bpf_map_update_elem(refcnt_map_fd, &key, value_data, 0);
+		if (ret)
+			goto unlock;
+	}
+
+	ret = *value_data;
+unlock:
+	xdp_lock_release(lock_fd);
+out:
+	free(value_data);
+	return ret;
+}
+
+static int xsk_incr_prog_refcnt(int refcnt_map_fd)
+{
+	return xsk_update_prog_refcnt(refcnt_map_fd, 1);
+}
+
+static int xsk_decr_prog_refcnt(int refcnt_map_fd)
+{
+	return xsk_update_prog_refcnt(refcnt_map_fd, -1);
+}
+
 static int __xsk_setup_xdp_prog(struct xsk_socket *xsk, int *xsks_map_fd)
 {
 	const char *fallback_prog = "xsk_def_xdp_prog_5.3.o";
 	const char *default_prog = "xsk_def_xdp_prog.o";
 	struct xsk_ctx *ctx = xsk->ctx;
-	const char *file_name;
+	const char *file_name = NULL;
 	bool attached = false;
 	int err;
 
@@ -688,6 +797,37 @@ static int __xsk_setup_xdp_prog(struct xsk_socket *xsk, int *xsks_map_fd)
 	if (IS_ERR(ctx->xdp_prog))
 		return PTR_ERR(ctx->xdp_prog);
 
+	ctx->refcnt_map_fd = -ENOENT;
+
+	if (ctx->xdp_prog) {
+		int refcnt;
+
+		ctx->refcnt_map_fd = xsk_lookup_refcnt_map(xdp_program__fd(ctx->xdp_prog), NULL);
+		if (ctx->refcnt_map_fd == -ENOENT)
+			goto map_lookup;
+
+		if (ctx->refcnt_map_fd < 0) {
+			err = ctx->refcnt_map_fd;
+			goto err_prog_load;
+		}
+
+		refcnt = xsk_incr_prog_refcnt(ctx->refcnt_map_fd);
+		if (refcnt < 0) {
+			err = refcnt;
+			pr_debug("Error occurred when incrementing xsk XDP prog refcount: %s\n",
+				 strerror(-err));
+			goto err_prog_load;
+		}
+
+		if (!refcnt) {
+			pr_warn("Current program is being detached, falling back on creating a new program\n");
+			close(ctx->refcnt_map_fd);
+			ctx->refcnt_map_fd = -ENOENT;
+			xdp_program__close(ctx->xdp_prog);
+			ctx->xdp_prog = NULL;
+		}
+	}
+
 	if (!ctx->xdp_prog) {
 		file_name = xsk_check_redirect_flags() ? default_prog : fallback_prog;
 		ctx->xdp_prog = xdp_program__find_file(file_name, NULL, NULL);
@@ -706,6 +846,15 @@ static int __xsk_setup_xdp_prog(struct xsk_socket *xsk, int *xsks_map_fd)
 		attached = true;
 	}
 
+	if (ctx->refcnt_map_fd < 0) {
+		ctx->refcnt_map_fd = xsk_lookup_refcnt_map(xdp_program__fd(ctx->xdp_prog),
+							   file_name);
+		if (ctx->refcnt_map_fd < 0 && ctx->refcnt_map_fd != -ENOENT) {
+			err = ctx->refcnt_map_fd;
+			goto err_prog_load;
+		}
+	}
+map_lookup:
 	ctx->xsks_map_fd = xsk_lookup_bpf_map(xdp_program__fd(ctx->xdp_prog));
 	if (ctx->xsks_map_fd < 0) {
 		err = ctx->xsks_map_fd;
@@ -727,12 +876,15 @@ err_lookup:
 		xdp_program__detach(ctx->xdp_prog, ctx->ifindex,
 				    xsk_convert_xdp_flags(xsk->config.xdp_flags), 0);
 err_prog_load:
+	if (ctx->refcnt_map_fd >= 0)
+		close(ctx->refcnt_map_fd);
+	ctx->refcnt_map_fd = -ENOENT;
 	xdp_program__close(ctx->xdp_prog);
 	ctx->xdp_prog = NULL;
 	return err;
 }
 
-static struct xsk_ctx *xsk_get_ctx(struct xsk_umem *umem, int ifindex, __u32 queue_id)
+static struct xsk_ctx *xsk_get_ctx(struct xsk_umem *umem, __u64 netns_cookie, int ifindex, __u32 queue_id)
 {
 	struct xsk_ctx *ctx;
 
@@ -740,7 +892,7 @@ static struct xsk_ctx *xsk_get_ctx(struct xsk_umem *umem, int ifindex, __u32 que
 		return NULL;
 
 	list_for_each_entry(ctx, &umem->ctx_list, list) {
-		if (ctx->ifindex == ifindex && ctx->queue_id == queue_id) {
+		if (ctx->netns_cookie == netns_cookie && ctx->ifindex == ifindex && ctx->queue_id == queue_id) {
 			ctx->refcount++;
 			return ctx;
 		}
@@ -776,7 +928,7 @@ out_free:
 }
 
 static struct xsk_ctx *xsk_create_ctx(struct xsk_socket *xsk,
-				      struct xsk_umem *umem, int ifindex,
+				      struct xsk_umem *umem, __u64 netns_cookie, int ifindex,
 				      const char *ifname, __u32 queue_id,
 				      struct xsk_ring_prod *fill,
 				      struct xsk_ring_cons *comp)
@@ -800,6 +952,7 @@ static struct xsk_ctx *xsk_create_ctx(struct xsk_socket *xsk,
 		memcpy(comp, umem->comp_save, sizeof(*comp));
 	}
 
+	ctx->netns_cookie = netns_cookie;
 	ctx->ifindex = ifindex;
 	ctx->refcount = 1;
 	ctx->umem = umem;
@@ -865,6 +1018,8 @@ int xsk_socket__create_shared(struct xsk_socket **xsk_ptr,
 	struct xsk_socket *xsk;
 	struct xsk_ctx *ctx;
 	int err, ifindex;
+	__u64 netns_cookie;
+	socklen_t optlen;
 	bool unmap;
 
 	if (!umem || !xsk_ptr || !(rx || tx))
@@ -896,14 +1051,24 @@ int xsk_socket__create_shared(struct xsk_socket **xsk_ptr,
 		tx_setup_done = umem->tx_ring_setup_done;
 	}
 
-	ctx = xsk_get_ctx(umem, ifindex, queue_id);
+	optlen = sizeof(netns_cookie);
+	err = getsockopt(xsk->fd, SOL_SOCKET, SO_NETNS_COOKIE, &netns_cookie, &optlen);
+	if (err) {
+		if (errno != ENOPROTOOPT) {
+			err = -errno;
+			goto out_socket;
+		}
+		netns_cookie = INIT_NS;
+	}
+
+	ctx = xsk_get_ctx(umem, netns_cookie, ifindex, queue_id);
 	if (!ctx) {
 		if (!fill || !comp) {
 			err = -EFAULT;
 			goto out_socket;
 		}
 
-		ctx = xsk_create_ctx(xsk, umem, ifindex, ifname, queue_id,
+		ctx = xsk_create_ctx(xsk, umem, netns_cookie, ifindex, ifname, queue_id,
 				     fill, comp);
 		if (!ctx) {
 			err = -ENOMEM;
@@ -1071,6 +1236,27 @@ int xsk_umem__delete(struct xsk_umem *umem)
 	return 0;
 }
 
+static void xsk_release_xdp_prog(struct xsk_socket *xsk)
+{
+	struct xsk_ctx *ctx = xsk->ctx;
+	int value;
+
+	if (xsk->ctx->refcnt_map_fd < 0)
+		goto out;
+
+	value = xsk_decr_prog_refcnt(ctx->refcnt_map_fd);
+	if (value < 0)
+		pr_warn("Error occurred when decrementing xsk XDP prog refcount: %s, please detach program yourself\n",
+			strerror(-value));
+	if (value)
+		goto out;
+
+	xdp_program__detach(ctx->xdp_prog, ctx->ifindex,
+			    xsk_convert_xdp_flags(xsk->config.xdp_flags), 0);
+out:
+	xdp_program__close(ctx->xdp_prog);
+}
+
 void xsk_socket__delete(struct xsk_socket *xsk)
 {
 	size_t desc_sz = sizeof(struct xdp_desc);
@@ -1086,7 +1272,7 @@ void xsk_socket__delete(struct xsk_socket *xsk)
 	umem = ctx->umem;
 	if (ctx->xdp_prog) {
 		xsk_delete_map_entry(ctx->xsks_map_fd, ctx->queue_id);
-		xdp_program__close(ctx->xdp_prog);
+		xsk_release_xdp_prog(xsk);
 	}
 
 	err = xsk_get_mmap_offsets(xsk->fd, &off);
