@@ -86,7 +86,7 @@ static int set_rlimit(unsigned int min_limit)
 	return 0;
 }
 
-int double_rlimit()
+int double_rlimit(void)
 {
 	pr_debug("Permission denied when loading eBPF object; "
 		 "raising rlimit and retrying\n");
@@ -354,6 +354,7 @@ int get_pinned_program(const struct iface *iface, const char *pin_root_path,
 	}
 
 	while ((de = readdir(dr)) != NULL) {
+		DECLARE_LIBXDP_OPTS(xdp_program_opts, opts, 0);
 		struct xdp_program *prog;
 
 		if (!strcmp(".", de->d_name) || !strcmp("..", de->d_name))
@@ -372,16 +373,17 @@ int get_pinned_program(const struct iface *iface, const char *pin_root_path,
 			continue;
 		}
 
-		prog = xdp_program__from_pin(pin_path);
-		if (IS_ERR_OR_NULL(prog) ||
+		opts.pin_path = pin_path;
+		prog = xdp_program__create(&opts);
+		if (libxdp_get_error(prog) ||
 		    !(m = xdp_program__is_attached(prog, iface->ifindex))) {
-			ret = IS_ERR(prog) ? PTR_ERR(prog) : -ENOENT;
+			ret = libxdp_get_error(prog) ?: -ENOENT;
 			pr_debug("Program %s no longer loaded on %s: %s\n",
 				 de->d_name, iface->ifname, strerror(-ret));
 			err = unlink(pin_path);
 			if (err)
 				ret = err;
-			if (!IS_ERR_OR_NULL(prog))
+			if (prog)
 				xdp_program__close(prog);
 		} else {
 			if (strcmp(xdp_program__name(prog), de->d_name)) {
@@ -608,13 +610,14 @@ out:
 	return mnt;
 }
 
-int get_bpf_root_dir(char *buf, size_t buf_len, const char *subdir)
+int get_bpf_root_dir(char *buf, size_t buf_len, const char *subdir, bool fatal)
 {
 	const char *bpf_dir;
 
 	bpf_dir = bpf_get_work_dir();
 	if (!bpf_dir) {
-		pr_warn("Could not find BPF working dir - bpffs not mounted?\n");
+		logging_print(fatal ? LOG_WARN : LOG_DEBUG,
+			      "Could not find BPF working dir - bpffs not mounted?\n");
 		return -ENOENT;
 	}
 
@@ -688,7 +691,7 @@ const char *action2str(__u32 action)
 	return NULL;
 }
 
-int check_bpf_environ(const char *pin_root_path)
+int check_bpf_environ(void)
 {
 	init_lib_logging();
 
@@ -697,12 +700,6 @@ int check_bpf_environ(const char *pin_root_path)
 		return 1;
 	}
 
-	if (!pin_root_path) {
-		pr_warn("Couldn't find a valid bpffs. "
-			"Please mount bpffs at %s\n", BPF_DIR_MNT);
-		return 1;
-	}
-
 	/* Try to avoid probing errors due to rlimit exhaustion by starting out
 	 * with an rlimit of 1 MiB. This is not going to solve all issues, but
 	 * it will at least make things work when there is nothing else loaded.
@@ -807,13 +804,13 @@ int prog_lock_get(const char *progname)
 			}
 
 			len = read(fd, buf, sizeof(buf) - 1);
+			err = -errno;
 			close(fd);
 			if (len > 0) {
 				buf[len] = '\0';
 				pid = strtoul(buf, NULL, 10);
 			}
-			if (!pid || errno) {
-				err = -errno;
+			if (!pid || err) {
 				pr_warn("Unable to read PID from lockfile: %s\n",
 					strerror(-err));
 				return err;
